#include <windows.h>
#include "win030.h"

/* Declaracin del procedimiento de ventana */
LRESULT CALLBACK WindowProcedure(HWND, UINT, WPARAM, LPARAM);
void TrazaEstilos(HDC);
void TrazaTramas(HDC);
void Cruz(HDC, HPEN, int, int);

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (hThisInstance, "Icono");
    wincl.hIconSm = LoadIcon (hThisInstance, "Icono");
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;                      /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;                      /* clase o la ventana */
    /* Usar el color de fondo por defecto para es escritorio */
    wincl.hbrBackground = GetStockObject(WHITE_BRUSH); //GetSysColorBrush(COLOR_BACKGROUND);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 030",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           500,
           500,
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/* Esta funcin es llamada por la funcin del API DispatchMessage() */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    HDC hdc;
    PAINTSTRUCT ps;
    static int modo;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           modo = CM_ESTILOS;
           break;
        case WM_PAINT:
           hdc = BeginPaint(hwnd, &ps);
           switch(modo) {
               case CM_ESTILOS:
                   TrazaEstilos(hdc);
                   break;
               case CM_TRAMAS:
                   TrazaTramas(hdc);
                   break;
           }
           EndPaint(hwnd, &ps);
           break;
        case WM_SIZE:
           InvalidateRect(hwnd, NULL, TRUE);
           break;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case CM_ESTILOS:
              case CM_TRAMAS:
                 modo = LOWORD(wParam);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
           }
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

void TrazaEstilos(HDC hdc) {
   HPEN pluma[4];
   LOGBRUSH lb[3] = {
      {BS_SOLID, RGB(240, 0, 0), 0},
      {BS_SOLID, RGB(0, 240, 0), 0},
      {BS_SOLID, RGB(0, 0, 240), 0}
   };

   pluma[0] = ExtCreatePen(
      PS_GEOMETRIC | PS_SOLID | PS_ENDCAP_ROUND | PS_JOIN_BEVEL,
      15, &lb[0], 0, NULL);
   pluma[1] = ExtCreatePen(
      PS_GEOMETRIC | PS_SOLID | PS_ENDCAP_SQUARE | PS_JOIN_MITER,
      15, &lb[1], 0, NULL);
   pluma[2] = ExtCreatePen(
      PS_GEOMETRIC | PS_SOLID | PS_ENDCAP_FLAT | PS_JOIN_ROUND,
      15, &lb[2], 0, NULL);
   pluma[3] = CreatePen(PS_SOLID, 0, RGB(0,0,0));

   SelectObject(hdc, pluma[0]);
   MoveToEx(hdc, 30, 30, NULL);
   LineTo(hdc, 400, 30);
   Rectangle(hdc, 30,120,400,300);

   SelectObject(hdc, pluma[1]);
   MoveToEx(hdc, 30, 60, NULL);
   LineTo(hdc, 400, 60);
   Rectangle(hdc, 50,140,380,280);

   SelectObject(hdc, pluma[2]);
   MoveToEx(hdc, 30, 90, NULL);
   LineTo(hdc, 400, 90);
   Rectangle(hdc, 70,160,360,260);

   Cruz(hdc, pluma[3], 30,30);
   Cruz(hdc, pluma[3], 30,60);
   Cruz(hdc, pluma[3], 30,90);
   Cruz(hdc, pluma[3], 400,30);
   Cruz(hdc, pluma[3], 400,60);
   Cruz(hdc, pluma[3], 400,90);

   Cruz(hdc, pluma[3], 30,120);
   Cruz(hdc, pluma[3], 400,120);
   Cruz(hdc, pluma[3], 30,300);
   Cruz(hdc, pluma[3], 400,300);

   Cruz(hdc, pluma[3], 50,140);
   Cruz(hdc, pluma[3], 380,140);
   Cruz(hdc, pluma[3], 50,280);
   Cruz(hdc, pluma[3], 380,280);

   Cruz(hdc, pluma[3], 70,160);
   Cruz(hdc, pluma[3], 360,160);
   Cruz(hdc, pluma[3], 70,260);
   Cruz(hdc, pluma[3], 360,260);

   DeleteObject(pluma[0]);
   DeleteObject(pluma[1]);
   DeleteObject(pluma[2]);
   DeleteObject(pluma[3]);
}

void TrazaTramas(HDC hdc) {
   int i;
   const int nPlumas = 11;
   HPEN pluma[nPlumas];
   LOGBRUSH lb[9] = {
      {BS_SOLID, RGB(240, 0, 0), 0},
      {BS_SOLID, RGB(0, 240, 0), 0},
      {BS_SOLID, RGB(0, 0, 240), 0},
      {BS_HATCHED, RGB(240,0,0),HS_BDIAGONAL},
      {BS_HATCHED, RGB(0,240,0),HS_CROSS},
      {BS_HATCHED, RGB(0,0,240),HS_DIAGCROSS},
      {BS_HATCHED, RGB(240,240,0),HS_FDIAGONAL},
      {BS_HATCHED, RGB(240,0,240),HS_HORIZONTAL},
      {BS_HATCHED, RGB(0,240,240),HS_VERTICAL}
   };
   DWORD trazos[8] = {10,20,20,20,30,20,40,20};

   pluma[0] = ExtCreatePen(
      PS_GEOMETRIC | PS_DASH | PS_ENDCAP_ROUND | PS_JOIN_ROUND,
      15, &lb[0], 0, NULL);
   pluma[1] = ExtCreatePen(
      PS_GEOMETRIC | PS_DOT | PS_ENDCAP_ROUND | PS_JOIN_ROUND,
      15, &lb[1], 0, NULL);
   pluma[2] = ExtCreatePen(
      PS_GEOMETRIC | PS_DASHDOT | PS_ENDCAP_ROUND | PS_JOIN_ROUND,
      15, &lb[2], 0, NULL);
   pluma[3] = ExtCreatePen(
      PS_GEOMETRIC | PS_DASHDOTDOT | PS_ENDCAP_ROUND | PS_JOIN_ROUND,
      15, &lb[0], 0, NULL);
   pluma[4] = ExtCreatePen(
      PS_GEOMETRIC | PS_USERSTYLE | PS_ENDCAP_ROUND | PS_JOIN_ROUND,
      15, &lb[1], 8, trazos);

   for(i = 5; i < nPlumas; i++)
      pluma[i] = ExtCreatePen(
         PS_GEOMETRIC | PS_SOLID | PS_ENDCAP_ROUND | PS_JOIN_ROUND,
         15, &lb[i-2], 0, NULL);

   for(i = 0; i < nPlumas; i++) {
      SelectObject(hdc, pluma[i]);
      MoveToEx(hdc, 30, 30+i*30, NULL);
      LineTo(hdc, 400, 30+i*30);
   }

   for(i = 0; i < nPlumas; i++) DeleteObject(pluma[i]);
}

void Cruz(HDC hdc, HPEN pluma, int x, int y) {
   SelectObject(hdc, pluma);

   MoveToEx(hdc, x, y-10, NULL);
   LineTo(hdc, x, y+10);
   MoveToEx(hdc, x-10, y, NULL);
   LineTo(hdc, x+10, y);
}

