#include <windows.h>
#include "win031.h"

/* Declaracin del procedimiento de ventana */
LRESULT CALLBACK WindowProcedure(HWND, UINT, WPARAM, LPARAM);
void CrearCaret(HWND, HBITMAP, int);
void PintaFondo(HDC, HWND, int);
void LlenarPantalla(HDC, HWND, HBITMAP);

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (hThisInstance, "Icono");
    wincl.hIconSm = LoadIcon (hThisInstance, "Icono");
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;                      /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;                      /* clase o la ventana */
    /* Usar el color de fondo por defecto para es escritorio */
    wincl.hbrBackground = GetSysColorBrush(COLOR_BACKGROUND);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 031",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           450,
           450,
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/* Esta funcin es llamada por la funcin del API DispatchMessage() */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    HDC hdc;
    PAINTSTRUCT ps;
    static HINSTANCE hInstance;
    static HBITMAP hBitmap;
    static UINT ActualBlink;
    static int tipo;
    static int posicion;
    static UINT parpadeo;
    static int fondo;
    RECT re;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
           hBitmap = LoadBitmap(hInstance, "Caret");
           parpadeo = ActualBlink = GetCaretBlinkTime();
           tipo = 0;
           posicion = 0;
           fondo = 0;
           break;
        case WM_SETFOCUS:
           CrearCaret(hwnd, hBitmap, tipo);
           ShowCaret(hwnd);
           break;
        case WM_KILLFOCUS:
           SetCaretBlinkTime(ActualBlink);
           DestroyCaret();
           break;
        case WM_PAINT:
           HideCaret(hwnd);
           hdc = BeginPaint(hwnd, &ps);
           PintaFondo(hdc, hwnd, fondo);
           EndPaint(hwnd, &ps);
           ShowCaret(hwnd);
           break;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case CM_VERTICAL:
              case CM_HORIZONTAL:
              case CM_SOLIDO:
              case CM_GRIS:
              case CM_BITMAP:
                 tipo = LOWORD(wParam)-CM_VERTICAL;
                 break;
              case CM_IZQUIERDA:
              case CM_CENTRO:
              case CM_DERECHA:
                 posicion = LOWORD(wParam)-CM_IZQUIERDA;
                 break;
              case CM_ACTUAL:
                 parpadeo = ActualBlink;
                 break;
              case CM_RAPIDO:
                 parpadeo = 100;
                 break;
              case CM_LENTO:
                 parpadeo = 1000;
                 break;
              case CM_LAZOS:
              case CM_POMPAS:
              case CM_FONDO:
                 fondo = LOWORD(wParam)-CM_LAZOS;
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
           }
           CrearCaret(hwnd, hBitmap, tipo);
           GetClientRect(hwnd, &re);
           switch(posicion) {
              case 0:
                 SetCaretPos(10,10);
                 break;
              case 1:
                 SetCaretPos(re.right/2, re.bottom/2);
                 break;
              case 2:
                 SetCaretPos(re.right-30, re.bottom-30);
                 break;
           }
           SetCaretBlinkTime(parpadeo);
           ShowCaret(hwnd);
           break;
        case WM_DESTROY:
           DeleteObject(hBitmap);
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

void CrearCaret(HWND hwnd, HBITMAP hBitmap, int tipo) {
   switch(tipo) {
      case 0:
         CreateCaret(hwnd, (HBITMAP)NULL, 0,20);
         break;
      case 1:
         CreateCaret(hwnd, (HBITMAP)NULL, 20,0);
         break;
      case 2:
         CreateCaret(hwnd, (HBITMAP)NULL, 20,20);
         break;
      case 3:
         CreateCaret(hwnd, (HBITMAP)1, 20,20);
         break;
      case 4:
         CreateCaret(hwnd, (HBITMAP)hBitmap, 20,0);
         break;
   }
}

void PintaFondo(HDC hdc, HWND hwnd, int fondo) {
    static HBITMAP hBitmap;

    switch(fondo) {
       case 0: // Lazos
          hBitmap = (HBITMAP)LoadImage(NULL, "lazo.bmp", IMAGE_BITMAP,
              0, 0, LR_LOADFROMFILE);
          LlenarPantalla(hdc, hwnd, hBitmap);
          DeleteObject(hBitmap);
          break;
       case 1: // Pompas
          hBitmap = (HBITMAP)LoadImage(NULL, "Pompas.bmp", IMAGE_BITMAP,
              0, 0, LR_LOADFROMFILE);
          LlenarPantalla(hdc, hwnd, hBitmap);
          DeleteObject(hBitmap);
          break;
       case 2: // Slido
          break;
    }
}

void LlenarPantalla(HDC hDC, HWND hWnd, HBITMAP hBitmap) {
    HDC memDC;
    BITMAP bm;
    int x, y;
    RECT re;

    memDC = CreateCompatibleDC(hDC);
    SelectObject(memDC, hBitmap);
    GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
    GetClientRect(hWnd, &re);
    y = 0;
    while(y < re.bottom) {
       x = 0;
       while(x < re.right) {
          BitBlt(hDC, x, y, bm.bmWidth, bm.bmHeight, memDC, 0, 0, SRCCOPY);
          x+=bm.bmWidth;
       }
       y+=bm.bmHeight;
    }
    DeleteDC(memDC);
}

