#include <windows.h>
#include <stdio.h>
#include "win036.h"

/* Declaracin del procedimiento de ventana */
LRESULT CALLBACK WindowProcedure(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DlgProc(HWND, UINT, WPARAM, LPARAM);

/* Declaraciones de tipos */
typedef struct stDatos {
   char Texto1[80];
   char Texto2[80];
} DATOS;

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (hThisInstance, "icono");
    wincl.hIconSm = LoadIcon (hThisInstance, "icono");
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;    /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;    /* clase o la ventana */
    /* Usar el color de fondo por defecto para es escritorio */
    wincl.hbrBackground =  GetSysColorBrush(COLOR_BACKGROUND); //GetStockObject(WHITE_BRUSH);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 036",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           500,
           450,
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/* Esta funcin es llamada por la funcin del API DispatchMessage() */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    HDC hdc;
    PAINTSTRUCT ps;
    static HINSTANCE hInstance;
    static DATOS Datos;
    char nombre[128], cad[256];
    RECT re;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
           strcpy(Datos.Texto1, "Inicial 1");
           strcpy(Datos.Texto2, "Inicial 2");
           break;
        case WM_ACTIVATE:
           if((HWND)lParam == NULL) strcpy(nombre, "NULL");
           else GetWindowText((HWND)lParam, nombre, 128);
           if(LOWORD(wParam) == WA_INACTIVE)
              sprintf(cad, "Ventana desactivada hacia %s", nombre);
           else
              sprintf(cad, "Ventana activada desde %s", nombre);
           hdc = GetDC(hwnd);
           SetBkColor(hdc, GetSysColor(COLOR_BACKGROUND));
           GetClientRect(hwnd, &re);
           re.top=30;
           re.bottom=58;
           FillRect(hdc, &re, GetSysColorBrush(COLOR_BACKGROUND));
           TextOut(hdc, 10, 30, cad, strlen(cad));
           ReleaseDC(hwnd, hdc);
           break;
        case WM_KILLFOCUS:
           if((HWND)wParam == NULL) strcpy(nombre, "NULL");
           else GetWindowText((HWND)wParam, nombre, 128);
           sprintf(cad, "Prdida de foco en favor de %s", nombre);
           hdc = GetDC(hwnd);
           SetBkColor(hdc, GetSysColor(COLOR_BACKGROUND));
           GetClientRect(hwnd, &re);
           re.top=10;
           re.bottom=28;
           FillRect(hdc, &re, GetSysColorBrush(COLOR_BACKGROUND));
           TextOut(hdc, 10, 10, cad, strlen(cad));
           ReleaseDC(hwnd, hdc);
           break;
        case WM_SETFOCUS:
           if((HWND)wParam == NULL) strcpy(nombre, "NULL");
           else GetWindowText((HWND)wParam, nombre, 128);
           sprintf(cad, "Foco recuperado desde %s", nombre);
           hdc = GetDC(hwnd);
           SetBkColor(hdc, GetSysColor(COLOR_BACKGROUND));
           GetClientRect(hwnd, &re);
           re.top=10;
           re.bottom=28;
           FillRect(hdc, &re, GetSysColorBrush(COLOR_BACKGROUND));
           TextOut(hdc, 10, 10, cad, strlen(cad));
           ReleaseDC(hwnd, hdc);
           break;
        case WM_COMMAND:
           if(LOWORD(wParam) == CM_DIALOGO)
              DialogBoxParam(hInstance, "DialogoPrueba",
                             hwnd, DlgProc, (LPARAM)&Datos);
           break;
        case WM_PAINT:
           hdc = BeginPaint(hwnd, &ps);
           EndPaint(hwnd, &ps);
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

BOOL CALLBACK DlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static DATOS *Datos;
    static BOOL cambio;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_INITDIALOG:
           SendDlgItemMessage(hDlg, ID_CONTROL1, EM_LIMITTEXT, 80, 0L);
           Datos = (DATOS *)lParam;
           SetDlgItemText(hDlg, ID_CONTROL1, Datos->Texto1);
           SetDlgItemText(hDlg, ID_CONTROL2, Datos->Texto2);
           cambio = FALSE;
           EnableWindow(GetDlgItem(hDlg, ID_CONTROL2), cambio);
           SetFocus(GetDlgItem(hDlg, ID_CONTROL1));
           return FALSE;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case ID_CAMBIO:
                 cambio = !cambio;
                 EnableWindow(GetDlgItem(hDlg, ID_CONTROL1), !cambio);
                 EnableWindow(GetDlgItem(hDlg, ID_CONTROL2), cambio);
                 if(cambio) SetFocus(GetDlgItem(hDlg, ID_CONTROL2));
                 else       SetFocus(GetDlgItem(hDlg, ID_CONTROL1));
                 break;
              case IDOK:
                 GetDlgItemText(hDlg, ID_CONTROL1, Datos->Texto1, 80);
                 GetDlgItemText(hDlg, ID_CONTROL2, Datos->Texto2, 80);
                 EndDialog(hDlg, FALSE);
                 break;
              case IDCANCEL:
                 EndDialog(hDlg, FALSE);
                 break;
           }
           return TRUE;
    }
    return FALSE;
}

