#include <windows.h>
#include <stdio.h>

typedef union {
   struct {
      unsigned int repeticion:16;
      unsigned int scan:8;
      unsigned int extendida:1;
      unsigned int reservado:4;
      unsigned int contexto:1;
      unsigned int previo:1;
      unsigned int transicion:1;
   };
   unsigned int lParam;
} keyData;

/* Declaracin del procedimiento de ventana */
LRESULT CALLBACK WindowProcedure(HWND, UINT, WPARAM, LPARAM);

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (hThisInstance, "icono");
    wincl.hIconSm = LoadIcon (hThisInstance, "icono");
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = NULL;
    wincl.cbClsExtra = 0;    /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;    /* clase o la ventana */
    /* Usar el color de fondo por defecto para es escritorio */
    wincl.hbrBackground =  GetSysColorBrush(COLOR_BACKGROUND); //GetStockObject(WHITE_BRUSH);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 037",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           500,
           450,
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        /* TranslateMessage(&mensaje); */
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/* Esta funcin es llamada por la funcin del API DispatchMessage() */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    HDC hdc;
    PAINTSTRUCT ps;
    char cad[256];
    static char lista[40][128];
    static int nLineas;
    int i;
    keyData kd;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           nLineas = 0;
           break;
        case WM_PAINT:
           hdc = BeginPaint(hwnd, &ps);
           SetBkColor(hdc, GetSysColor(COLOR_BACKGROUND));
           for(i = 0; i < nLineas; i++)
              TextOut(hdc, 10, i*20, lista[i], strlen(lista[i]));
           EndPaint(hwnd, &ps);
           break;
        case WM_KEYDOWN:
           for(i = nLineas; i > 0; i--)
              strcpy(lista[i], lista[i-1]);
           if(nLineas < 39) nLineas++;
           kd.lParam = lParam;
           GetKeyNameText(lParam, cad, 256);
           sprintf(lista[0], "Tecla %s (%d) pulsada Rep=%d "
              "[Ext:%d Ctx:%d Prv:%d Trn:%d]",
              cad, (int)wParam, kd.repeticion, kd.extendida,
              kd.contexto, kd.previo, kd.transicion);
           InvalidateRect(hwnd, NULL, TRUE);
           break;
        case WM_KEYUP:
           for(i = nLineas; i > 0; i--)
              strcpy(lista[i], lista[i-1]);
           if(nLineas < 39) nLineas++;
           kd.lParam = lParam;
           GetKeyNameText(lParam, cad, 256);
           sprintf(lista[0], "Tecla %s (%d) liberada "
              "[Ext:%d Ctx:%d Prv:%d Trn:%d]",
              cad, (int)wParam, kd.extendida,
              kd.contexto, kd.previo, kd.transicion);
           InvalidateRect(hwnd, NULL, TRUE);
           break;
        case WM_SYSKEYDOWN:
           for(i = nLineas; i > 0; i--)
              strcpy(lista[i], lista[i-1]);
           if(nLineas < 39) nLineas++;
           kd.lParam = lParam;
           GetKeyNameText(lParam, cad, 256);
           sprintf(lista[0], "Tecla de sistema %s (%d) pulsada Rep=%d"
              " [Ext:%d Ctx:%d Prv:%d Trn:%d]",
              cad, (int)wParam, kd.repeticion, kd.extendida,
              kd.contexto, kd.previo, kd.transicion);
           InvalidateRect(hwnd, NULL, TRUE);
           return DefWindowProc(hwnd, msg, wParam, lParam);
           break;
        case WM_SYSKEYUP:
           for(i = nLineas; i > 0; i--)
              strcpy(lista[i], lista[i-1]);
           if(nLineas < 39) nLineas++;
           kd.lParam = lParam;
           GetKeyNameText(lParam, cad, 256);
           sprintf(lista[0], "Tecla de sistema %s (%d) liberada "
               "[Ext:%d Ctx:%d Prv:%d Trn:%d]",
              cad, (int)wParam, kd.extendida,
              kd.contexto, kd.previo, kd.transicion);
           InvalidateRect(hwnd, NULL, TRUE);
           return DefWindowProc(hwnd, msg, wParam, lParam);
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}


