#include <windows.h>

#include "win052.h"

/*  Declaracin del procedimiento de ventana  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DlgProc(HWND, UINT, WPARAM, LPARAM);
void ActivarEstilo(HWND, int);

/* Declaraciones de tipos */
typedef struct stDatos {
   char Texto[80];
   int Estado;
} DATOS;

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;                      /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;                      /* clase o la ventana */
    /* Usar el color de fondo por defecto para la ventana */
    wincl.hbrBackground = GetSysColorBrush(COLOR_BACKGROUND);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 052",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           544,                 /* Ancho */
           375,                 /* Alto en pixels */
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/*  Esta funcin es invocada por la funcin DispatchMessage()  */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static HINSTANCE hInstance;
    /* Variables para dilogo */
    static DATOS Datos;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
           /* Inicializacin de los datos de la aplicacin */
           strcpy(Datos.Texto, "Inicial");
           Datos.Estado = 0;
           return 0;
           break;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
               case CM_DIALOGO:
                   DialogBoxParam(hInstance, "DialogoPrueba", hwnd, DlgProc, (LPARAM)&Datos);
                   break;
               case CM_SALIR:
                   DestroyWindow(hwnd);
                   break;
           }
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

BOOL CALLBACK DlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static DATOS *Datos;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_INITDIALOG:
           SendDlgItemMessage(hDlg, ID_TEXTO, EM_LIMITTEXT, 80, 0L);
           Datos = (DATOS *)lParam;
           SetDlgItemText(hDlg, ID_TEXTO, Datos->Texto);
           /* Activar el estilo segn el valor de estado */
           ActivarEstilo(GetDlgItem(hDlg, ID_TEXTO), Datos->Estado);
           /* Activar el radiobutton */
           CheckRadioButton(hDlg, ID_RADIOBUTTON1, ID_RADIOBUTTON3,
              ID_RADIOBUTTON1+Datos->Estado);
           SetFocus(GetDlgItem(hDlg, ID_TEXTO));
           return FALSE;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case ID_RADIOBUTTON1:
              case ID_RADIOBUTTON2:
              case ID_RADIOBUTTON3:
                 Datos->Estado = LOWORD(wParam)-ID_RADIOBUTTON1;
                 ActivarEstilo(GetDlgItem(hDlg, ID_TEXTO), Datos->Estado);
                 SetFocus(GetDlgItem(hDlg, ID_TEXTO));
                 break;
              case IDOK:
                 GetDlgItemText(hDlg, ID_TEXTO, Datos->Texto, 80);
                 EndDialog(hDlg, FALSE);
                 break;
              case IDCANCEL:
                 EndDialog(hDlg, FALSE);
                 break;
           }
           return TRUE;
    }
    return FALSE;
}

void ActivarEstilo(HWND hctrl, int estado)
{
   LONG estiloActual;

   estiloActual = GetWindowLong(hctrl, GWL_STYLE);
   switch(estado) {
      case 0: /* Normal */
         estiloActual &= ~ES_UPPERCASE;
         estiloActual &= ~ES_LOWERCASE;
         break;
      case 1: /* Maysculas */
         estiloActual &= ~ES_LOWERCASE;
         estiloActual |= ES_UPPERCASE;
         break;
      case 2: /* Minsculas */
         estiloActual &= ~ES_UPPERCASE;
         estiloActual |= ES_LOWERCASE;
         break;
   }
   SetWindowLong(hctrl, GWL_STYLE, estiloActual);
}
