#include <windows.h>
#include <stdio.h>

#include "win054.h"

/*  Declaracin del procedimiento de ventana  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
void ActualizarMenu(HWND, HWND);
void AsignarTexto(HWND, char*);
void Guardar(HWND hctrl, char *fichero);

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;                      /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;                      /* clase o la ventana */
    /* Usar el color de fondo por defecto para la ventana */
    wincl.hbrBackground = GetSysColorBrush(COLOR_BACKGROUND);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 054",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           544,                 /* Ancho */
           375,                 /* Alto en pixels */
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/*  Esta funcin es invocada por la funcin DispatchMessage()  */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static HINSTANCE hInstance;
    static HWND hctrl;
    static HFONT hfont;
    RECT re;
    int inicio, final;
    BOOL modif;
    char mensaje[512];
    DWORD lista[10] = {10,25,40,65,95,130,160,200,250,360};

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
           /* Insertar control Edit */
           GetClientRect(hwnd, &re);
           hctrl = CreateWindowEx(
              0,
              "EDIT",          /* Nombre de la clase */
              "",              /* Texto del ttulo, no tiene */
              ES_LEFT | ES_MULTILINE | ES_WANTRETURN |
              WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP |
              WS_HSCROLL | WS_VSCROLL, /* Estilo */
              0, 0,            /* Posicin */
              re.right, re.bottom, /* Tamao */
              hwnd,            /* Ventana padre */
              (HMENU)ID_TEXTO, /* Identificador del control */
              hInstance,       /* Instancia */
              NULL);           /* Sin datos de creacin de ventana */
           hfont = CreateFont(24, 0, 0, 0, 300,
              FALSE, FALSE, FALSE, DEFAULT_CHARSET,
              OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS,
              PROOF_QUALITY, DEFAULT_PITCH | FF_ROMAN,
              "Times New Roman");
           //hfont = (HFONT)GetStockObject( DEFAULT_GUI_FONT );
           SendMessage(hctrl, WM_SETFONT, (WPARAM)hfont, MAKELPARAM(TRUE, 0));
           SendMessage(hctrl, EM_SETMARGINS,
              EC_LEFTMARGIN | EC_RIGHTMARGIN, MAKELONG(50, 30));
           /* Inicializacin de los datos de la aplicacin */
           AsignarTexto(hctrl, "texto.txt");
           ActualizarMenu(hwnd, hctrl);
           SetFocus(hctrl);
           return 0;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case CM_SALIR:
                 DestroyWindow(hwnd);
                 break;
              case CM_GUARDAR:
                 Guardar(hctrl, "texto.txt");
                 SendMessage(hctrl, EM_SETMODIFY, FALSE, 0);
                 ActualizarMenu(hwnd, hctrl);
                 break;
              case ID_TEXTO:
                 if(HIWORD(wParam) == EN_CHANGE)
                    if(SendMessage(hctrl, EM_GETMODIFY, 0, 0))
                       EnableMenuItem(GetMenu(hwnd), CM_GUARDAR, MF_BYCOMMAND | MF_ENABLED);
                 break;
              case WM_CUT:
              case WM_COPY:
              case WM_PASTE:
              case WM_CLEAR:
              case EM_UNDO:
                 SendMessage(hctrl, LOWORD(wParam), 0, 0);
                 break;
              case CM_GETSEL:
                 SendMessage(hctrl, EM_GETSEL, (WPARAM)&inicio, (LPARAM)&final);
                 sprintf(mensaje, "Seleccin actual de %d a %d", inicio, final);
                 MessageBox(hwnd, mensaje, "Ejemplo 54", MB_OK);
                 break;
              case CM_SETSEL:
                 SendMessage(hctrl, EM_SETSEL, 1500, 1559);
                 break;
              case CM_SETSEL2:
                 SendMessage(hctrl, EM_SETSEL, 1500, 1559);
                 SendMessage(hctrl, EM_SCROLLCARET, 0, 0);
                 break;
              case CM_REPLACE:
                 strcpy(mensaje, "TEXTO SUSTITUIDO");
                 SendMessage(hctrl, EM_REPLACESEL, TRUE, (LPARAM)mensaje);
                 break;
              case CM_MODIFICADO:
                 modif = SendMessage(hctrl, EM_GETMODIFY, 0, 0);
                 if(modif) strcpy(mensaje, "Texto modificado");
                 else strcpy(mensaje, "Texto no modificado");
                 MessageBox(hwnd, mensaje, "Ejemplo 54", MB_OK);
                 break;
              case CM_TABULAR10:
                 lista[0] = 10;
                 SendMessage(hctrl, EM_SETTABSTOPS, 1, (LPARAM)&lista);
                 break;
              case CM_TABULAR20:
                 lista[0] = 20;
                 SendMessage(hctrl, EM_SETTABSTOPS, 1, (LPARAM)&lista);
                 break;
              case CM_TABULAR:
                 lista[0] = 10;
                 SendMessage(hctrl, EM_SETTABSTOPS, 10, (LPARAM)&lista);
                 break;
              case CM_LINESCROLL:
                 SendMessage(hctrl, EM_LINESCROLL, 20, 10);
                 break;
              case CM_PAGINAARRIBA:
                 SendMessage(hctrl, EM_SCROLL, SB_PAGEUP, 0);
                 break;
              case CM_LINEAARRIBA:
                 SendMessage(hctrl, EM_SCROLL, SB_LINEUP, 0);
                 break;
              case CM_LINEAABAJO:
                 SendMessage(hctrl, EM_SCROLL, SB_LINEDOWN, 0);
                 break;
              case CM_PAGINAABAJO:
                 SendMessage(hctrl, EM_SCROLL, SB_PAGEDOWN, 0);
                 break;
           }
           break;
        case WM_SIZE:
           GetClientRect(hwnd, &re);
           MoveWindow(hctrl, 0, 0, re.right, re.bottom, TRUE);
           GetClientRect(hctrl, &re);
           re.left += 20;
           re.top += 20;
           re.right -= 20;
           re.bottom -= 40;
           SendMessage(hctrl, EM_SETRECT, 0, (LPARAM)&re);
           break;
        case WM_DESTROY:
           DeleteObject(hfont);
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

void ActualizarMenu(HWND hwnd, HWND hctrl) {
   if(SendMessage(hctrl, EM_GETMODIFY, 0, 0))
      EnableMenuItem(GetMenu(hwnd), CM_GUARDAR, MF_BYCOMMAND | MF_ENABLED);
   else
      EnableMenuItem(GetMenu(hwnd), CM_GUARDAR, MF_BYCOMMAND | MF_GRAYED);
}

void AsignarTexto(HWND hctrl, char* fichero) {
   FILE *fe;
   char cad[1024];
   char* buffer;
   int l;
   int i;
   char separador[4] = {'\r', '\r', '\n', 0};
   HLOCAL hloc;

   fe = fopen(fichero, "r");
   /* Calcular el tamao del buffer:
      eliminar los caracteres de control al final de cada
      lnea y sumar tres (ruptura de lnea blanda \r\r\n) */
   l = 0;
   do {
      fgets(cad, 1024, fe);
      if(!feof(fe)) {
         /* No contar los caracteres especiales al final */
         i = strlen(cad)-1;
         while(i>=0 && cad[i] < ' ') i--;
         l += i + 4; /* Sumar tres caracteres por lnea.
            Sumamos el valor 4 porque i contiene la longitud de
            la cadena menos 1 */
      }
   } while (!feof(fe));

   hloc = (HLOCAL)SendMessage(hctrl, EM_GETHANDLE, 0, 0);
   LocalFree(hloc);
   hloc = LocalAlloc(LMEM_MOVEABLE, l+1);

   buffer = (char *)LocalLock(hloc);

   /* Crear buffer */
   //buffer = (char *)malloc(l+1);
   buffer[0] = 0;

   /* Hacer la lectura */
   rewind(fe);
   do {
      fgets(cad, 1024, fe);
      if(!feof(fe)) {
         /* Eliminar los caracteres especiales al final */
         i = strlen(cad)-1;
         while(i>=0 && cad[i] < ' ') cad[i--] = 0;
         strcat(buffer, cad);
         strcat(buffer, separador);
      }
   } while (!feof(fe));

   fclose(fe);
   LocalUnlock(hloc);
   SendMessage(hctrl, EM_SETHANDLE, (WPARAM)hloc, 0);
   // SendMessage(hctrl, WM_SETTEXT, 0, (LPARAM)buffer);
   // free(buffer);
}

void Guardar(HWND hctrl, char *fichero) {
   FILE *fs;
   int nLineas, longitud, i;
   char linea[1024];

   fs = fopen(fichero, "w");
   if(fs) {
      nLineas = SendMessage(hctrl, EM_GETLINECOUNT, 0, 0);
      for(i = 0; i < nLineas; i++) {
         *(WORD*)linea = 1024;
         longitud = SendMessage(hctrl, EM_GETLINE, (WPARAM)i, (LPARAM)linea);
         linea[longitud] = 0;
         fprintf(fs, "%s\n", linea);
      }
      fclose(fs);
   }
}
