#include <windows.h>
#include <stdio.h>
#include "win065.h"

/*  Declaracin del procedimiento de ventana  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
void IniciarLista(HWND);
int CalculaLongitud(HWND, char *);

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;                      /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;                      /* clase o la ventana */
    /* Usar el color de fondo por defecto para la ventana */
    wincl.hbrBackground = GetSysColorBrush(COLOR_BACKGROUND);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 065",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           470,                 /* Ancho */
           300,                 /* Alto en pixels */
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/*  Esta funcin es invocada por la funcin DispatchMessage()  */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static HINSTANCE hInstance;
    HFONT hfont;
    static HWND hctrl;
    int i;
    char cad[128];
    RECT re;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
           /* Insertar control Edit */
           hctrl = CreateWindowEx(
              0,
              "LISTBOX",       /* Nombre de la clase */
              "",              /* Texto del ttulo, no tiene */
              LBS_STANDARD | WS_HSCROLL | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP, /* Estilo */
              9, 25,           /* Posicin */
              444, 220,        /* Tamao */
              hwnd,            /* Ventana padre */
              (HMENU)ID_LISTA, /* Identificador del control */
              hInstance,       /* Instancia */
              NULL);           /* Sin datos de creacin de ventana */
           /* Inicializacin de los datos de la aplicacin */
           IniciarLista(hctrl);
           SendMessage(hctrl, LB_SETCURSEL, (WPARAM)0, 0);
           hfont = (HFONT)GetStockObject( DEFAULT_GUI_FONT );
           SendMessage(hctrl, WM_SETFONT, (WPARAM)hfont, MAKELPARAM(TRUE, 0));
           //SendMessage(hctrl, LB_SETHORIZONTALEXTENT, 840, 0);
           SetFocus(hctrl);
           return 0;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case CM_VERALTURA:
                i = SendMessage(hctrl, LB_GETITEMHEIGHT, 0, 0);
                sprintf(cad, "Altura es %d", i);
                MessageBox(hwnd, cad, "Altura de tem", MB_OK);
                break;
              case CM_ALTURA10:
                SendMessage(hctrl, LB_SETITEMHEIGHT, 0, MAKELPARAM(10,0));
                InvalidateRect(hctrl, NULL, TRUE);
                break;
              case CM_ALTURA20:
                SendMessage(hctrl, LB_SETITEMHEIGHT, 0, MAKELPARAM(20,0));
                InvalidateRect(hctrl, NULL, TRUE);
                break;
              case CM_ALTURA40:
                SendMessage(hctrl, LB_SETITEMHEIGHT, 0, MAKELPARAM(40,0));
                InvalidateRect(hctrl, NULL, TRUE);
                break;
              case CM_RECTANGULO:
                i = SendMessage(hctrl, LB_GETCURSEL, 0, 0);
                SendMessage(hctrl, LB_GETITEMRECT, i, (LPARAM)&re);
                sprintf(cad, "(%ld, %ld) - (%ld, %ld)", re.left, re.top,
                  re.right, re.bottom);
                MessageBox(hwnd, cad, "Rectngulo de tem", MB_OK);
                break;
              case CM_ITEMPUNTO:
                i = SendMessage(hctrl, LB_ITEMFROMPOINT, 0, MAKELPARAM(40, 123));
                sprintf(cad, "Item en (40,123) es %d", i);
                MessageBox(hwnd, cad, "Item de un punto", MB_OK);
                break;
              case CM_SALIR:
                DestroyWindow(hwnd);
                break;
           }
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

void IniciarLista(HWND hlista)
{
   char cad[512];
   int i, x;
   int eActual;

   eActual = SendMessage(hlista, LB_GETHORIZONTALEXTENT, 0, 0);

   for(i = 0; i < 50; i++) {
      sprintf(cad, "tem %05d de 50 de una anchura tal que no cabe en "
        "el list box que hemos definido, o al menos no debera caber, "
        "si las cosas salen tal y como las hemos calculado, claro.", i);
      x = CalculaLongitud(hlista, cad);
      if(x > eActual) eActual = x;
      SendMessage(hlista, LB_ADDSTRING, 0, (LPARAM)cad);
   }
   SendMessage(hlista, LB_SETHORIZONTALEXTENT, eActual, 0);
}

int CalculaLongitud(HWND hwnd, char *cad)
{
   HDC hdc;
   SIZE tam;
   HFONT hfont;

   hfont = (HFONT)GetStockObject( DEFAULT_GUI_FONT );
   hdc = GetDC(hwnd);
   SelectObject(hdc, hfont);
   GetTextExtentPoint32(hdc, cad, strlen(cad), &tam);
   /* LPtoDP(hdc, (POINT *)&tam, 1); */
   ReleaseDC(hwnd, hdc);

   return tam.cx;
}
