#include <windows.h>
#include <winspool.h>
#include <commdlg.h>

#define CM_IMPRIMIR 102

/*  Declaracin del procedimiento de ventana  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
void ImprimirDocumento(PRINTDLG *);
void ImprimirPagina(HDC, int, int);

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */


    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;                      /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;                      /* clase o la ventana */
    /* Usar el color de fondo por defecto para la ventana */
    wincl.hbrBackground =  GetSysColorBrush(COLOR_MENU); //COLOR_BACKGROUND);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 079",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           544,                 /* Ancho */
           455,                 /* Alto en pixels */
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
       if(!IsDialogMessage(hwnd, &mensaje) ) {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
        }
    }
    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/*  Esta funcin es invocada por la funcin DispatchMessage()  */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    /* Variables para dilogo */
    static HINSTANCE hInstance;
    DEVMODE *dm;
    PRINTDLG pd = {sizeof(PRINTDLG),
        hwnd, 0, 0,
        0, // hDC
        PD_ALLPAGES | PD_RETURNDC, // | PD_USEDEVMODECOPIESANDCOLLATE,
        1, 2,
        1, 2,
        1,  // Copias
        NULL, 0, 0, 0, 0, 0, 0, 0};
    char cad[256];

    switch (msg)              /* manipulador del mensaje */
    {
        case WM_CREATE:
           hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
           CreateWindowEx(
              0,
              "BUTTON",      /* Nombre de la clase */
              "Imprimir",           /* Texto del ttulo */
              BS_PUSHBUTTON | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP, /* Estilo */
              9, 12,           /* Posicin */
              500, 400,         /* Tamao */
              hwnd,           /* Ventana padre */
              (HMENU)CM_IMPRIMIR, /* Identificador del control */
              hInstance,      /* Instancia */
              NULL);          /* Sin datos de creacin de ventana */
           return 0;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case CM_IMPRIMIR:
                if(PrintDlg(&pd)) {
                    dm = GlobalLock(pd.hDevMode);
                    wsprintf(cad, "Impresora: %s\nDimensiones: %dx%d mm\nTipo: %s\nRes: %dx%d\nDesde: %d\nHasta: %d\nCopias: %d\nIntercalar: %s",
                        dm->dmDeviceName, dm->dmPaperWidth, dm->dmPaperLength, dm->dmColor==DMCOLOR_COLOR ? "Color" : "B/W", dm->dmPrintQuality,
                        dm->dmYResolution, pd.nFromPage, pd.nToPage,
                        pd.nCopies, (pd.Flags & PD_COLLATE) ? "si" : "no");
                    MessageBox(hwnd, cad, "Selecciones", MB_OK);
                    GlobalUnlock(pd.hDevMode);
                    ImprimirDocumento(&pd);
                    GlobalFree(pd.hDevMode);
                    GlobalFree(pd.hDevNames);
                    DeleteDC(pd.hDC);
                }
                break;
           }
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

void ImprimirDocumento(PRINTDLG *pd) {
    DOCINFO di;
    int pagina;
    int copia;
    int nCopias;
    int repetirPagina;

    di.cbSize = sizeof(DOCINFO);
    di.lpszDocName = "Prueba";
    di.lpszOutput = NULL;
    di.lpszDatatype = NULL;
    di.fwType = 0;

    if(pd->Flags & PD_COLLATE) {
        // Intercalar las copias, imprimimos nCopies documentos:
        nCopias = pd->nCopies;
        repetirPagina = 1;
    } else {
        // No intercalar las copias, un documento con nCopies de cada pgina:
        nCopias = 1;
        repetirPagina = pd->nCopies;
    }
    for(copia=0; copia < nCopias; copia++) {
        pagina=1;

        StartDoc(pd->hDC, &di);
        if(pagina >= pd->nFromPage && pagina <= pd->nToPage) ImprimirPagina(pd->hDC, 1, repetirPagina);
        pagina++;
        if(pagina >= pd->nFromPage && pagina <= pd->nToPage) ImprimirPagina(pd->hDC, 2, repetirPagina);
        EndDoc(pd->hDC);
    }
}

void ImprimirPagina(HDC hDC, int npag, int ncopias) {
    HBITMAP hBitmap;
    HDC memDC;
    BITMAP bm;
    int ancho, alto;
    int i;

    // Dimensiones del dispositivo de salida en pixels:
    ancho = GetDeviceCaps(hDC, HORZRES);
    alto = GetDeviceCaps(hDC, VERTRES);

    // Cargar mapa de bits para pagina 2:
    hBitmap = (HBITMAP)LoadImage(NULL, "meninas24.bmp", IMAGE_BITMAP,
        0, 0, LR_LOADFROMFILE);
    GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);

    for(i = 0; i < ncopias; i++) {
        // Empezar pgina:
        StartPage(hDC);

        switch(npag) {
            case 1:
                TextOut(hDC, 10, 10, "Hola, mundo!", 12);
                Ellipse(hDC, 150, 180, ancho-150, alto-180);
                break;
            case 2:
                memDC = CreateCompatibleDC(hDC);
                SelectObject(memDC, hBitmap);
                StretchBlt(hDC, 300, 300, ancho-600, alto-600, memDC, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY);
                DeleteDC(memDC);
        }
        EndPage(hDC);
    }

    // Liberar mapa de bits:
    DeleteObject(hBitmap);
}
